/******************************************************************************/
/*  			Copyright Spark Fun Electronics                               */
/******************************************************************************/
/*                                                                            */
/*  gps.c:  functions for the Lassen IQ GPS in NMEA mode                      */
/*                                                                            */
/*  3-20-05: Removed Lat/Lon averaging to fix "warmup" bug.                   */
/*                                                                            */
/******************************************************************************/

//*******************************************************
//					Libraries
//*******************************************************
#include "gps.h"
#include <stdio.h>

#include "serial.h"
#include "rprintf.h"

//User setting storage
#include "iap_flash.h"

//Needed for uint8_t
#include <stdint.h>

#include <stdlib.h>
#include <string.h>


//*******************************************************
//					Global Variables for gps.c
//    last two bytes after the * are the checksum
//    of the string between '$' and '*'
//    char str[] = "PSRF103,04,00,10,00";
//    unsigned char ival = 0;
//    for(int i = 0; i < strlen(str);i++ ) ival ^= str[i];
//    printf("%x\n",ival );
//
//*******************************************************
const char gps_init_string[]="$PSRF103,01,00,00,01*25\r\n"	//Turn off GLL messages
"$PSRF103,02,00,00,01*26\r\n"	//Turn off GSA messages
"$PSRF103,03,00,00,01*27\r\n"	//Turn off GSV messages
"$PSRF103,05,00,00,01*21\r\n"	//Turn off VTG messages
"$PSRF103,06,00,00,01*22\r\n"	//Turn off ??? messages
"$PSRF103,07,00,00,01*23\r\n"	//Turn off ??? messages
"$PSRF103,00,00,00,01*24\r\n"	//Turn off GGA messages
"$PSRF103,00,00,01,00*24\r\n"	//Set CGA messages to 1 Hz rate with checksum disabled!
"$PSRF103,04,00,00,01*20\r\n"	//Turn off RMC messages
"$PSRF103,04,00,10,00*20\r\n";	//Set RMC messages to 1 Hz rate with checksum enabled
//"$PSRF103,04,00,00,01*20\r\n"	//Turn off RMC messages
//"$PSRF103,04,00,01,00*20\r\n";	//Set RMC messages to 1 Hz rate with checksum enabled

int locks = 0;

//*******************************************************
//					GPS Functions
//*******************************************************
//Usage: GPS_init_strings();
//Inputs: None
//Outputs: None
//Description: This function will initialize the GPS module
//				with the strings defined in the gps_init_string
//				array defined in the Global Varibales section.
void GPS_init_strings(void)
{
    for(int i=0; gps_init_string[i] != '\0'; i++)
    {
        putc_serial0(gps_init_string[i]);
    }
}

/*
void do_parse(void)
{
    char outbuf[100];

    if(strcmp(field[0],"$GPRMC")==0)
    {
			// Found GPRMC
            // Field 0 = $GPRMC
            // Field 1 = UTC hhmmss.sss
            // Latitude ddmm.mmmm
            // N/S
            // Longitude dddmm.mmmm
            // E/W
            // Speed  in knots
            // course over ground in degrees
            // Date ddmmyy
            // Mag variation (blank)
            // Checksum


        if(*field[2] == 'A')
        {
            GPS_set_time(field[1],field[9]);

            locks++;
            if(locks >= 10)
            {
                locks = 10;
                // Locked
                lattof(field[3],field[4]);
                lontof(field[5],field[6]);
                fix_quality=1;
                speed_kh_fix = atof(field[7])*1.8520;
                if(speed_kh_fix < 1.50)
                {
                    speed_kh_fix = 0.0;
                }
                sprintf(outbuf,"G, %f, %f, %f, %s, %s\n", latitude_fix
                        , longitude_fix
                        , speed_kh_fix
                        , gps_time
                        , gps_date);
                rprintf("\n\rGPS Stats=%s\n", outbuf);
                //        LOG_string(outbuf);
            }
            else
            {
                rprintf("\n\rT, GPS reporting lock, but unstable.\n");
                //        LOG_string("T, GPS reporting lock, but unstable.\n");
            }
        }
        else
        {
            // Not Locked
            fix_quality=0;
            rprintf("\n\rT, GPS is out of lock.\n");
            //      LOG_string("T, GPS is out of lock.\n");

            // *** right here is where we should set locks to 0 if we're gonna....
            //       * We'll do it-- but it's not clear that we should

            locks = 0;
        }
    }

    //NES : Added to look for Sats in view
    if(strcmp(field[0],"$GPGGA")==0)
    {
			// Found GPGGA
            // Field 0 = $GPGGA
            // Field 1 = UTC hhmmss.sss
            // Latitude ddmm.mmmm
            // N/S
            // Longitude dddmm.mmmm
            // E/W
            // Position Fix indicator
            // Satellites Used
            // blah
            //
        if(*field[6] == '1')// || *field[6] == '2') //DGPS fix is 2
        {
            numberSats = field[7][1] - '0';
            //rprintf("\n\rSats in view = %d\n", numberSats);
        }
        else
        {
            numberSats = 0;
            //rprintf("\n\rNo GPS Lock\n");
        }
    }
}
*/
/*
void lattof(char* lat, char* ns)
{
    int i;
    char tempCalc[32];

    latitude_fix = 0;
    tempCalc[0] = lat[0];
    tempCalc[1] = lat[1];
    tempCalc[2] = 0x00;

    // Latitude degrees in first two chars
    latitude_fix = atof(tempCalc);

    / Then get minutes and decimal minutes
    for (i = 0; lat[i + 2] != 0x00; i++)
    {
        tempCalc[i] = lat[i + 2];
    }

    // Then turn minutes to decimal degrees
    tempCalc[i + 1] = 0x00;
    latitude_fix += atof(tempCalc) / 60.0;
    lati[latidx]=latitude_fix;
    latidx=(latidx+1)&3;

    // And correct for North / South
    if(ns[0] == 'S')
    {
        latitude_fix *= -1.0;
    }
}

void lontof(char* lon, char* ew)
{
    int i;
    char tempCalc[32];

    longitude_fix = 0;

    // First three numbers are degrees
    tempCalc[0] = lon[0];
    tempCalc[1] = lon[1];
    tempCalc[2] = lon[2];
    tempCalc[3] = 0x00;
    longitude_fix = atof(tempCalc);

    // Then the remaining chars are minutes and
    // decimal minutes
    for (i = 0; lon[i + 3] != 0x00; i++)
    {
        tempCalc[i] = lon[i + 3];
    }

    tempCalc[i + 1] = 0x00;

    // Convert to degrees
    longitude_fix += atof(tempCalc) / 60.0;
    longi[longidx]=longitude_fix;
    longidx=(longidx+1)&3;

    // Correct for East/West
    if(ew[0] == 'W')
    {
        longitude_fix *= -1.0;
    }
}
*/
/*
void GPS_set_time(char* time_field, char* date_field)
{
    gps_time[0]=time_field[0];
    gps_time[1]=time_field[1];
    gps_time[3]=time_field[2];
    gps_time[4]=time_field[3];
    gps_time[6]=time_field[4];
    gps_time[7]=time_field[5];

    gps_date[0]=date_field[0];
    gps_date[1]=date_field[1];
    gps_date[3]=date_field[2];
    gps_date[4]=date_field[3];
    gps_date[6]=date_field[4];
    gps_date[7]=date_field[5];

	//Correct this date/time with the current GPS offset
	uint8_t hour_offset;
	hour_offset = GPS_get_offset();

	uint8_t c_hour, c_minute, c_second;
	uint8_t c_year, c_month, c_day;

	c_hour = (gps_time[0] - '0') * 10;
	c_hour += (gps_time[1] - '0');
	c_minute = (gps_time[3] - '0') * 10;
	c_minute += (gps_time[4] - '0');
	c_second = (gps_time[6] - '0') * 10;
	c_second += (gps_time[7] - '0');

	c_day = (gps_date[0] - '0') * 10;
	c_day += (gps_date[1] - '0');
	c_month = (gps_date[3] - '0') * 10;
	c_month += (gps_date[4] - '0');
	c_year = (gps_date[6] - '0') * 10;
	c_year += (gps_date[7] - '0');

	//Correct for UTC and local time offset
	if(c_hour < hour_offset)
	{
		c_hour += 24;
		c_day -= 1;

		//Now subtract a day from the date
		if(c_day == 0)
		{
			//If this is the first day of the month, reset to the last day of the previous month
			if( (c_month==1) | (c_month==3) | (c_month==5) | (c_month==7) | (c_month==8) | (c_month==10) | (c_month==12))
				c_day = 31;

			if( (c_month==4) | (c_month==6) | (c_month==9) | (c_month==11))
				c_day = 30;

			if(c_month == 2)
				c_day = 82;

			c_month--;
			if(c_month == 0)
			{
				c_month = 12;
				c_year--;
			}
		}
	}
	c_hour -= hour_offset;

	//Convert back to string
	sprintf(gps_time, "%.2d:%.2d:%.2d", c_hour, c_minute, c_second);
	sprintf(gps_date, "%.2d/%.2d/%.2d", c_day, c_month, c_year);

	//rprintf("gps time=%s\n", gps_time);
	//rprintf("gps date=%s\n", gps_date);
}
*/
