#include <stdlib.h>
#include <stdio.h>
#include "parse.h"

int main(int argc, char *argv[] ) {
    char recType;
    unsigned char hour, min, sec, day, month, year;
    unsigned int secFrac;
    unsigned int   numTicks;
    char accelX, accelY, accelZ;
    char offsetX, offsetY, offsetZ;
    unsigned char numSat;
    short latDegree, longDegree;
    unsigned int latFrac, longFrac;
    float hdop, altitude;
    short sval;
    int   ival;
    char  signature[4];

    unsigned long int freq = 0L;
    char lastSec=0;
    unsigned int numSeconds =0;

    FILE *fd = NULL;
    char *fname = NULL;

    if ( argc > 1 ) {
		fname = argv[1];

	} else {
		printf("%s filename\n", argv[0] );
		return -1;
	}

    fd = fopen(fname, "rb" );
	if ( NULL == fd ) {
		printf("Error opening [%s]\n", fname );
		return -1;
	}

    fread(signature, sizeof(char),4, fd );
    if (('K' != signature[0])||('M' != signature[1])||('0' != signature[2])||('1' != signature[3])) {
        printf("Incorrect input file\n");
        fclose(fd);
        return -1;
	}

    while (!feof(fd)) {
        fread(&recType, sizeof(char),1, fd );
        if ( !feof(fd)) {
            switch(recType) {
                case 'G': //Gga Data
                    parseFullGGARecord( &fd, &hour, &min, &sec,
					                    &secFrac, &numSat,
					                    &hdop, &altitude,
					                    &latDegree, &latFrac,
                                        &longDegree, &longFrac );
                    break;
                case 'R':
                    parseFullRMARecord( &fd, &day, &month, &year,
					                    &hour, &min,&sec,
					                    &secFrac,
					                    &latDegree, &latFrac,
					                    &longDegree, &longFrac );
                    break;
                case 'g':
                    parsePartialGGARecord( &fd, &hour, &min, &sec,
                                           &secFrac, &numSat,
                                           &hdop, &altitude );
                    break;
                case 'r':
                    parsePartialRMARecord( &fd, &day, &month, &year,
                                           &hour, &min, &sec, &secFrac );
                    break;
                case 'A': //Time + Acceleration Data
                    parseFullAccelRecord( &fd, &hour, &min, &sec,
                                          &secFrac, &numTicks,
                                          &accelX, &accelY, &accelZ );

                    printf("%02d:%02d:%02d.%04d\t",hour,min,sec,secFrac );
                    printf("%d\t%d\t%d\n",accelX+offsetX, accelY+offsetY,accelZ+offsetZ );
                    break;
                case 'a': //Acceleration Data
                    parsePartialAccelRecord( &fd, &accelX, &accelY, &accelZ );
                    printf("::\t");
                    printf("%d\t%d\t%d\n",accelX+offsetX, accelY+offsetY,accelZ+offsetZ );
                    break;
                case 'C': //Config Info
                    parseConfigRecord( &fd, &offsetX, &offsetY, &offsetZ );
                    break;
                default:
                    break;
            }
        }
	}

	return 0;
}