#include <stdlib.h>
#include <stdio.h>
#include "parse.h"

int main(int argc, char *argv[] ) {
    char recType;
    unsigned char hour, min, sec, day, month, year;
    unsigned int secFrac;
    unsigned int   numTicks;
    char accelX, accelY, accelZ;
    char offsetX, offsetY, offsetZ;
    unsigned char numSat;
    short latDegree, longDegree;
    unsigned int latFrac, longFrac;
    float hdop, altitude;
    short sval;
    int   ival;
    char  signature[4];

    unsigned long int freq = 0L;
    char lastSec=0;
    unsigned int numSeconds =0;

    FILE *fd = NULL;
    char *fname = NULL;

    if ( argc > 1 ) {
		fname = argv[1];

	} else {
		printf("%s filename\n", argv[0] );
		return -1;
	}

    fd = fopen(fname, "rb" );
	if ( NULL == fd ) {
		printf("Error opening [%s]\n", fname );
		return -1;
	}

    fread(signature, sizeof(char),4, fd );
    if (('K' != signature[0])||('M' != signature[1])||('0' != signature[2])||('1' != signature[3])) {
        printf("Incorrect input file\n");
        fclose(fd);
        return -1;
	}

    day   = 0;
    month = 0;
    year  = 0;

    //First run to get a date and time
    while (!feof(fd)) {
        fread(&recType, sizeof(char),1, fd );
        if (!feof(fd)) {
            switch(recType) {
                case 'G': //Gga Data
                    parseFullGGARecord( &fd, &hour, &min, &sec,
					                    &secFrac, &numSat,
					                    &hdop, &altitude,
					                    &latDegree, &latFrac,
                                        &longDegree, &longFrac );
                   break;
                case 'R':
                    parseFullRMARecord( &fd, &day, &month, &year,
					                    &hour, &min,&sec,
					                    &secFrac,
					                    &latDegree, &latFrac,
					                    &longDegree, &longFrac );
					return 0;
                    break;
                case 'g':
                    parsePartialGGARecord( &fd, &hour, &min, &sec,
                                           &secFrac, &numSat,
                                           &hdop, &altitude );
                    break;
                case 'r':
                    parsePartialRMARecord( &fd, &day, &month, &year,
                                           &hour, &min, &sec, &secFrac );
                    break;
                case 'A': //Time + Acceleration Data
                    parseFullAccelRecord( &fd, &hour, &min, &sec,
                                          &secFrac, &numTicks,
                                          &accelX, &accelY, &accelZ );
                    break;
                case 'a': //Acceleration Data
                     parsePartialAccelRecord( &fd, &accelX, &accelY, &accelZ );
                    break;
                case 'C': //Config Info
                    parseConfigRecord( &fd, &offsetX, &offsetY, &offsetZ );
                    break;
                default:
                    break;
            }
        }
        if ( (unsigned char)0 < day ) break;
	}

    if ( (unsigned char)0 == day ) {
		printf("No GPS record found\n");
		return -1;
	}

    printf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
    printf("<kml xmlns=\"http://www.opengis.net/kml/2.2\">\n");
    printf("<Document>\n");
    printf("<name>my Path (%02d/%02d/%02d %02d:%02d:%02d) </name>\n", month,day,year,hour,min,sec);
    printf("<open>1</open>\n");
    printf("<Placemark>\n");
    printf("<name>%02d/%02d/%02d %02d:%02d:%02d </name>\n", month,day,year,hour,min,sec);
    printf("<visibility>1</visibility>\n");
    printf("<description>\n");
    printf("Path recorded with KinetaMap\n");
    printf("</description>\n");
    printf("<LineString id=\"1\">\n");
    printf("<extrude>0</extrude>\n");
    printf("<tesselate>1</tesselate>\n");
    printf("<altitudeMode>clampToGround</altitudeMode>\n");
    printf("<coordinates>\n");


    (void)fseek(fd, 4L, SEEK_SET);

    while (!feof(fd)) {
        fread(&recType, sizeof(char),1, fd );
        if ( !feof(fd)) {
            switch(recType) {
                case 'G': //Gga Data
                    parseFullGGARecord( &fd, &hour, &min, &sec,
					                    &secFrac, &numSat,
					                    &hdop, &altitude,
					                    &latDegree, &latFrac,
                                        &longDegree, &longFrac );
                    if (( -90 <= latDegree ) && ( 90 >= latDegree ) &&
                        ( -180 <= longDegree ) && ( 180 >= longDegree ))
                        printf("  %d.%05d,%d.%05d,0\n",longDegree, longFrac,latDegree, latFrac );

                    break;
                case 'R':
                    parseFullRMARecord( &fd, &day, &month, &year,
					                    &hour, &min,&sec,
					                    &secFrac,
					                    &latDegree, &latFrac,
					                    &longDegree, &longFrac );
                    if (( -90 <= latDegree ) && ( 90 >= latDegree ) &&
                        ( -180 <= longDegree ) && ( 180 >= longDegree ))
                        printf("  %d.%05d,%d.%05d,0\n",longDegree, longFrac,latDegree, latFrac );
                    break;
                case 'g':
                    parsePartialGGARecord( &fd, &hour, &min, &sec,
                                           &secFrac, &numSat,
                                           &hdop, &altitude );
                    break;
                case 'r':
                    parsePartialRMARecord( &fd, &day, &month, &year,
                                           &hour, &min, &sec, &secFrac );
                    break;
                case 'A': //Time + Acceleration Data
                    parseFullAccelRecord( &fd, &hour, &min, &sec,
                                          &secFrac, &numTicks,
                                          &accelX, &accelY, &accelZ );
                    break;
                case 'a': //Acceleration Data
                     parsePartialAccelRecord( &fd, &accelX, &accelY, &accelZ );
                    break;
                case 'C': //Config Info
                    parseConfigRecord( &fd, &offsetX, &offsetY, &offsetZ );
                    break;
                default:
                    break;
            }
        }
	}

    printf("</coordinates>\n");
    printf("</LineString>\n");
    printf("</Placemark>\n");
    printf("</Document>\n");
    printf("</kml>\n");

	return 0;
}