#include <stdio.h>
#ifdef _WINDOWS
#include <winsock.h>
#else
#include <arpa/inet.h>
#endif
#include "parse.h"


int parseConfigRecord( FILE **fd, char *const offsetX, char *const offsetY, char *const offsetZ ) {
    fread( offsetX, sizeof(char),1, *fd );
    fread( offsetY, sizeof(char),1, *fd );
    fread( offsetZ, sizeof(char),1, *fd );

    return( ferror(*fd));
}

int parseFullAccelRecord( FILE **fd, unsigned char *const hour, unsigned char *const min, unsigned char *const sec,
                          unsigned int *const secFrac, unsigned int *const numTicks,
                          char *const accelX, char *const accelY, char *const accelZ ) {
	short sval;
	int ival;

    fread( hour, sizeof(char),1, *fd );
    fread( min, sizeof(char),1, *fd );
    fread( sec, sizeof(char),1, *fd );
    //fread(&sval, sizeof(short),1, *fd );
    //secFrac = ntohs(sval);

    fread(&ival, sizeof(int),1, *fd );
    *numTicks = ntohl(ival);

    *secFrac = (*numTicks*10000)/65537;

    fread( accelX, sizeof(char),1, *fd );
    fread( accelY, sizeof(char),1, *fd );
    fread( accelZ, sizeof(char),1, *fd );

    return( ferror(*fd));

}

int parsePartialAccelRecord( FILE **fd, char *const accelX, char *const accelY, char *const accelZ ) {
    fread( accelX, sizeof(char),1, *fd );
    fread( accelY, sizeof(char),1, *fd );
    fread( accelZ, sizeof(char),1, *fd );

    return( ferror(*fd));
}

int parseFullGGARecord( FILE **fd, unsigned char *const hour, unsigned char *const min, unsigned char *const sec,
                        unsigned int *const secFrac, unsigned char *const numSat,
                        float *const hdop, float *const altitude,
                        short *const latDegree, unsigned int *const latFrac,
                        short *const longDegree, unsigned int *const longFrac ){
	short sval;
	int ival;

    fread( numSat, sizeof(char),1, *fd );
    fread(&sval, sizeof(short),1, *fd );
    sval = (short) ntohs(sval);
    *hdop = (float)sval/(float)10.0;
    fread(&sval, sizeof(short),1, *fd );
    sval = (short) ntohs(sval);
    *altitude = (float) sval/(float)10.0;

    fread( hour, sizeof(char),1, *fd );
    fread( min, sizeof(char),1, *fd );
    fread( sec, sizeof(char),1, *fd );
    fread(&sval, sizeof(short),1, *fd );
    *secFrac = (int) ntohs(sval);

    fread(&sval, sizeof(short),1, *fd );
    *latDegree = ntohs(sval);
    fread(&ival, sizeof(int),1, *fd );
    *latFrac = ntohl(ival);

    fread(&sval, sizeof(short),1, *fd );
    *longDegree = ntohs(sval);
    fread(&ival, sizeof(int),1, *fd );
    *longFrac = ntohl(ival);

    return( ferror(*fd));
}

int parsePartialGGARecord( FILE **fd, unsigned char *const hour, unsigned char *const min, unsigned char *const sec,
                           unsigned int *const secFrac, unsigned char *const numSat,
                           float *const hdop, float *const altitude ) {
	short sval;

    fread( numSat, sizeof(char),1, *fd );
    fread(&sval, sizeof(short),1, *fd );
    sval = (short) ntohs(sval);
    *hdop = (float)sval/(float)10.0;
    fread(&sval, sizeof(short),1, *fd );
    sval = (short) ntohs(sval);
    *altitude = (float) sval/(float)10.0;

    fread( hour, sizeof(char),1, *fd );
    fread( min, sizeof(char),1, *fd );
    fread( sec, sizeof(char),1, *fd );
    fread(&sval, sizeof(short),1, *fd );
    *secFrac = (int) ntohs(sval);

    return( ferror(*fd));
}

int parseFullRMARecord( FILE **fd, unsigned char *const day, unsigned char *const month, unsigned char *const year,
                        unsigned char *const hour, unsigned char *const min, unsigned char *const sec,
                        unsigned int *const secFrac,
                        short *const latDegree, unsigned int *const latFrac,
                        short *const longDegree, unsigned int *const longFrac ){
	short sval;
	int ival;

    fread( day, sizeof(char),1, *fd );
    fread( month, sizeof(char),1, *fd );
    fread( year, sizeof(char),1, *fd );

    fread( hour, sizeof(char),1, *fd );
    fread( min, sizeof(char),1, *fd );
    fread( sec, sizeof(char),1, *fd );
    fread(&sval, sizeof(short),1, *fd );
    *secFrac = (int) ntohs(sval);

    fread(&sval, sizeof(short),1, *fd );
    *latDegree = ntohs(sval);
    fread(&ival, sizeof(int),1, *fd );
    *latFrac = ntohl(ival);

    fread(&sval, sizeof(short),1, *fd );
    *longDegree = ntohs(sval);
    fread(&ival, sizeof(int),1, *fd );
    *longFrac = ntohl(ival);

    return( ferror(*fd));
}

int parsePartialRMARecord( FILE **fd, unsigned char *const day, unsigned char *const month, unsigned char *const year,
                           unsigned char *const hour, unsigned char *const min, unsigned char *const sec,
                           unsigned int *const secFrac ){
	short sval;
	int ival;

    fread( day, sizeof(char),1, *fd );
    fread( month, sizeof(char),1, *fd );
    fread( year, sizeof(char),1, *fd );

    fread( hour, sizeof(char),1, *fd );
    fread( min, sizeof(char),1, *fd );
    fread( sec, sizeof(char),1, *fd );
    fread(&sval, sizeof(short),1, *fd );
    *secFrac = (int) ntohs(sval);

    return( ferror(*fd));
}
