
#define BINARY_ONLY 1

//*******************************************************
//					Kineata Map Firmware
//*******************************************************
#include <stdio.h>
#include <string.h>

#include "LPC214x.h"
#include "serial.h"
#include "rprintf.h"
#include "KinetaMapPins.h"
#include "spi0.h"
#include "delay.h"

//*******************************************************
//				Memory Management Libraries
//*******************************************************
#include "rootdir.h"
#include "sd_raw.h"
#include "fat16.h"
#include "main_msc.h"

//*******************************************************
//					External Component Libs
//*******************************************************
#include "LIS302.h"
#include "gps.h"
#include "parseGps.h"

//*******************************************************
// The following define the size of the buffers used
// to store the data.
//*******************************************************
#define	GPS_BUFFER_SIZE	80

#define BUFFER_SIZE 1024
#define MAX_BUFFER_SIZE BUFFER_SIZE-100

//#define BINARY_BUFFER_SIZE 1024
//#define MAX_BINARY_BUFFER_SIZE BINARY_BUFFER_SIZE-100
#define BINARY_BUFFER_SIZE 16384
#define MAX_BINARY_BUFFER_SIZE BINARY_BUFFER_SIZE-100


//*******************************************************
//					Core Functions
//*******************************************************
void bootUp(void);
static void timer0ISR(void);
static void RxData0ISR(void);
static void EINT1ISR(void);
void reset(void);
void sendParsedData(void);
int get_adc_1(char channel);
void createLogFile(void);
void saveData(struct fat16_file_struct **fd, const char * const buf, const int buf_size);
signed int atoi(const char *const s);
void configBluetooth(void);
void load_settings(void);
void initializeGps(void);
void initializeBluetooth(void);
void initializeAccelerometer(char gRange );
void initializeTimer(int freqDivider);
int configMenu(void);

//*******************************************************
//					Global Variables
//*******************************************************
int baud_rate0 = 4800, baud_rate1 = 115200;
char bt_fastmode=0, gps_message_complete=0, valid_gps_data=0;
int gps_message_index=0, gps_message_size=0;
int final_gps_message_size=0;
char final_gps_message[GPS_BUFFER_SIZE], gps_message[GPS_BUFFER_SIZE];
GPSdata gpsData;

//Mark that the Real TIme Clock was not set
char RTC_Set = 0;
//Mark if we have been forced to sleep mode
char sleepMode = INACTIVE;


signed char acceleration_x, acceleration_y, acceleration_z;
char accel_read_complete, batt_read_complete;

int battery_level;

#ifndef BINARY_ONLY
struct fat16_file_struct * LOG_FILE = NULL;
char   log_data[BUFFER_SIZE];
int    log_data_index=0;

struct fat16_file_struct * NMEA_FILE = NULL;
char   nmea_data[BUFFER_SIZE];
int    nmea_data_index=0;
#endif

struct fat16_file_struct * BINARY_FILE = NULL;
char   binary_data[BINARY_BUFFER_SIZE];
int    binary_data_index=0;

char config_menu_setting;
char batt_logging, gps_over_bt;
int  accel_logging;
unsigned char gps_logging;

//Binary data is stored in network byte order
char binary_logging, txt_logging, nmea_logging;
signed char offsetX, offsetY, offsetZ;
signed char gRange;
int menu_option=0;

unsigned long int accel_hour, accel_min, accel_sec, accel_ctc;

//log_data_index += (int)sprintf( &(log_data[log_data_index++]), "[%x]=%c", );

int main (void)
{
//*******************************************************
//					Local Variables
//*******************************************************
    int i = 0;
	int j = 0;
	char c;

//*******************************************************
//					Main Code
//*******************************************************
	//GPSon();

	//Initialize ARM I/O
	bootUp(); //Init. I/O ports, Comm protocols and interrupts
    VICIntEnClr = UART0_INT | TIMER0_INT;	//Stop all running interrupts

    BlueOff();
	GreenOn();

	//Get the settings from the config file
    //A Default file will be created if one is not found
	load_settings();

	//Initialize the GPS Receiver
	//GPS is initialized even if it is not used, to get the initial date and time
	initializeGps();
    gpsData.date.day = UCHAR_MAX;

    //accel_logging gives the frequency Divider for the interrupts used to
    //collect accelleration data
    initializeTimer(accel_logging);

    //gRange is used to specify if the full scale measurement range should be
    //used.
    if ( 0 < accel_logging ) {
    	initializeAccelerometer( gRange );
	}

	if ((ACTIVE==config_menu_setting) || (ACTIVE==gps_over_bt)) {

		initializeBluetooth();

		if (config_menu_setting==ACTIVE){
			BlueOn();
			while((IOPIN1 & BT_CON)!=BT_CON);		//If the config menu option is active, wait until we get a BT connection
			while(((IOPIN1 & BT_CON)==BT_CON) && (config_menu_setting==ACTIVE)){
				menu_option=configMenu();
				if(menu_option=='9')break;
			}

			load_settings();
			//DIsable bluetooth if it was disabled via the UI
			if ((ACTIVE!=config_menu_setting) && (ACTIVE!=gps_over_bt)) IOCLR0 = BT_EN;

			BlueOff();
		}
    }

	initializeGps();//Repeat to ensure it worked
    enable_gps_rmc_msgs(1);//Enable the RMC message at the highest frequency to get initial date and time

    //accel_logging gives the frequency Divider for the interrupts used to
    //collect accelleration data
    initializeTimer(accel_logging);

    //gRange is used to specify if the full scale measurement range should be
    //used.
    if ( 0 < accel_logging ) {
    	initializeAccelerometer( gRange );
	}

	if(ACTIVE==binary_logging || ACTIVE==nmea_logging || ACTIVE==txt_logging)createLogFile();

	delay_ms(1000);
	GreenOff();

	sleepMode=INACTIVE;
    //Start UART0 interrupts(Print all received characters from UART0 to UART1)
	VICIntEnable = UART0_INT | TIMER0_INT | EINT1_INT;	//Enable UART0 and Timer0 Interrupts

	//Start Logging Data, the process will continue untill the power is removed or
	//an USB cable is connected.
    //
	while(1){

		//If a USB Cable gets plugged in, stop everything!
		if(IOPIN0 & (1<<23))
		{
			BlueOn();
            VICIntEnClr = UART0_INT | TIMER0_INT | EINT1_INT;	//Stop all running interrupts
            //Save any leftover data
    		if ( NULL != BINARY_FILE ) {
      	        saveData(&BINARY_FILE, binary_data, binary_data_index);
			    fat16_close_file(BINARY_FILE);
				binary_data_index=0;
			}
#ifndef BINARY_ONLY
    		if ( NULL != LOG_FILE ) {
      	        saveData(&LOG_FILE, log_data, log_data_index);
			    fat16_close_file(LOG_FILE);
				log_data_index=0;
			}
    		if ( NULL != NMEA_FILE ) {
      	        saveData(&NMEA_FILE, nmea_data, nmea_data_index);
			    fat16_close_file(NMEA_FILE);
				nmea_data_index=0;
			}
#endif
   		    sd_raw_sync();
			GreenOff();
			BlueOff();
            sleepMode = INACTIVE;
			main_msc();	//Open the mass storage device
			reset();	//Reset to check for new FW
		}

		//If there is a new GPS message available, copy it and set the current time
		if(gps_message_complete==1){
            VICIntEnClr = UART0_INT | TIMER0_INT;

			for(i=0, j=0; i<gps_message_size; i++){
			    final_gps_message[j++]=gps_message[i];
				gps_message[i]='\0';
			}
			final_gps_message[j] = '\0';
			final_gps_message_size=j-1;
			gps_message_complete=0;

#ifndef BINARY_ONLY
            if ((INACTIVE==sleepMode) && (ACTIVE==nmea_logging)) {
				for(i=0; i<=final_gps_message_size; i++){
   				    nmea_data[nmea_data_index++] = final_gps_message[i];
				}
				nmea_data[nmea_data_index++] = '\n';

				if(nmea_data_index >= MAX_BUFFER_SIZE){
 			        VICIntEnClr = TIMER0_INT | EINT1_INT;
					saveData(&NMEA_FILE, nmea_data, nmea_data_index);
					nmea_data_index=0;
                    VICIntEnable = TIMER0_INT | EINT1_INT;
				}
			}
#endif
			VICIntEnable = UART0_INT | TIMER0_INT | EINT1_INT;
            valid_gps_data=0;
			gpsData.date.day = UCHAR_MAX;
			if ( -1 != parseNmea(final_gps_message, final_gps_message_size, &gpsData )) {
				//If the clock was never set, do it now
				//When not in sleeping mode We set the clock only once so we can use the
				//clock tick to mark the accelerometer samples.
				//Only if we are in sleepmode we continue updating the
				//clock to synch it w/ gps

				if ((0 == RTC_Set) || (ACTIVE==sleepMode)){
                    VICIntEnClr = UART0_INT | TIMER0_INT | EINT1_INT;	//Stop all running interrupts
					BlueOn();

					//Set RTC
					CCR &= ~(1<<0);	//Disable the RTC
					CCR |= (1<<1); 	//Reset the RTC
					HOUR = gpsData.time.hour;
					MIN  = gpsData.time.min;
					SEC  = gpsData.time.sec;
					//DOM    = gpsData.date.day;
					//MONTH  = gpsData.date.month;
					//YEAR   = gpsData.date.year;
					CCR &= ~(1<<1);	//UnReset the RTC
					CCR |= (1<<0);		//Enable the Clock

                    VICIntEnable = TIMER0_INT | EINT1_INT;	//Enable interrupts

					if (0 == RTC_Set) {
  					    disable_gps_rmc_msgs();
						//delay_ms(200);

						gps_message_index=0;//discard any RMC data left in the queue
  					    if ( 0 < gps_logging ) {
							//If we are configured to receive gps at a given
							//interval we can now disable the RMC and use the GGA
							enable_gps_gga_msgs((unsigned char)gps_logging);
						}
						RTC_Set=1;
//					    VICIntEnable = UART0_INT;
					}
                    VICIntEnable = UART0_INT | TIMER0_INT | EINT1_INT;	//Enable all running interrupts

					if (ACTIVE==sleepMode) delay_ms(150);
					BlueOff();
				}
    			if(VALID_GPS(gpsData)) valid_gps_data=1;

				if ( gps_over_bt==ACTIVE ) {
					for(i=0; final_gps_message[i] != '\0'; i++){
						if((IOPIN1 & BT_CON)==BT_CON ) rprintf("%c", final_gps_message[i]); //If BT is connected, send the GPS message
					}
					rprintf("\n");
				}
			}
		}

		if((INACTIVE==sleepMode) && (((0<gps_logging) && (0<final_gps_message_size)) || (0<accel_logging && accel_read_complete) || (batt_logging==ACTIVE && batt_read_complete))) {
		    VICIntEnClr = TIMER0_INT;
			if (ACTIVE==binary_logging) {
			    if ((0<gps_logging) && (0<final_gps_message_size)) {
					if ( UCHAR_MAX != gpsData.date.day ) {
						//Received an RMC Record
						if ( 1==valid_gps_data) binary_data[binary_data_index++] = 'R';
						else binary_data[binary_data_index++] = 'r';

						binary_data[binary_data_index++] = gpsData.date.day;
						binary_data[binary_data_index++] = gpsData.date.month;
						binary_data[binary_data_index++] = gpsData.date.year;
					} else {
						if ( 1==valid_gps_data) {
							binary_data[binary_data_index++] = 'G';
							binary_data[binary_data_index++] = gpsData.numSatellites;
							//Already in NetworkByte Order
							c = (gpsData.hdop & 0xff00) >> 8;
							binary_data[binary_data_index++] = c;
							c = (gpsData.hdop & 0x00ff);
							binary_data[binary_data_index++] = c;
							c = (gpsData.altitude & 0xff00) >> 8;
							binary_data[binary_data_index++] = c;
							c = (gpsData.altitude & 0x00ff);
							binary_data[binary_data_index++] = c;
						}
						else binary_data[binary_data_index++] = 'g';

					}

					binary_data[binary_data_index++] = gpsData.time.hour;
					binary_data[binary_data_index++] = gpsData.time.min;
					binary_data[binary_data_index++] = gpsData.time.sec;

					//Already in NetworkByte Order
					c = (gpsData.time.thousand & 0xff00) >> 8;
					binary_data[binary_data_index++] = c;
					c = (gpsData.time.thousand & 0x00ff);
					binary_data[binary_data_index++] = c;

					if ( 1==valid_gps_data) {
						c = (gpsData.latitude.degree & 0xff00) >> 8;
						binary_data[binary_data_index++] = c;
						c = (gpsData.latitude.degree & 0x00ff);
						binary_data[binary_data_index++] = c;

						c = (gpsData.latitude.fractional & 0xff000000) >> 24;
						binary_data[binary_data_index++] = c;
						c = (gpsData.latitude.fractional & 0x00ff0000) >> 16;
						binary_data[binary_data_index++] = c;
						c = (gpsData.latitude.fractional & 0x0000ff00) >> 8;
						binary_data[binary_data_index++] = c;
						c = (gpsData.latitude.fractional& 0x000000ff);
						binary_data[binary_data_index++] = c;

						c = (gpsData.longitude.degree & 0xff00) >> 8;
						binary_data[binary_data_index++] = c;
						c = (gpsData.longitude.degree & 0x00ff);
						binary_data[binary_data_index++] = c;

						c = (gpsData.longitude.fractional & 0xff000000) >> 24;
						binary_data[binary_data_index++] = c;
						c = (gpsData.longitude.fractional & 0x00ff0000) >> 16;
						binary_data[binary_data_index++] = c;
						c = (gpsData.longitude.fractional & 0x0000ff00) >> 8;
						binary_data[binary_data_index++] = c;
						c = (gpsData.longitude.fractional& 0x000000ff);
						binary_data[binary_data_index++] = c;
					}
				}
				if (0<accel_logging && accel_read_complete) {
					if ( RTC_Set ) {
						binary_data[binary_data_index++] = 'A';
						//Write the time when the accelleration was logged
						c = (unsigned char) accel_hour;
						binary_data[binary_data_index++] = c;
						c = (unsigned char) accel_min;
						binary_data[binary_data_index++] = c;
						c = (unsigned char) accel_sec;
						binary_data[binary_data_index++] = c;

						//binary_data[binary_data_index++] = 0x00;//Fractional portion
						//binary_data[binary_data_index++] = 0x00;//not used

						//Write the Clock Tick Count collected when the accelleration was logged
						c = ((int)accel_ctc & 0xff000000) >> 24;
						binary_data[binary_data_index++] = c;
						c = ((int)accel_ctc & 0x00ff0000) >> 16;
						binary_data[binary_data_index++] = c;
						c = ((int)accel_ctc & 0x0000ff00) >> 8;
						binary_data[binary_data_index++] = c;
						c = ((int)accel_ctc & 0x000000ff);
						binary_data[binary_data_index++] = c;
					} else {
						binary_data[binary_data_index++] = 'a';
					}
					binary_data[binary_data_index++] = acceleration_x;

					binary_data[binary_data_index++] = acceleration_y;

					binary_data[binary_data_index++] = acceleration_z;

				}

				//Save Data if the buffer is full
				if(binary_data_index >= MAX_BINARY_BUFFER_SIZE){
					VICIntEnClr =  UART0_INT | EINT1_INT;
					GreenOn();
					saveData(&BINARY_FILE, binary_data, binary_data_index);
					binary_data_index=0;
					GreenOff();
					VICIntEnable =  UART0_INT | EINT1_INT;
				}
            }

#ifndef BINARY_ONLY
            if(ACTIVE==txt_logging) {
				//Log Time
				//If there is GPS data, use this time,
                if ((UCHAR_MAX!=gpsData.time.hour) && (0<final_gps_message_size)) {
  			        log_data_index += (int)sprintf( &(log_data[log_data_index++]), "%02d:%02d:%02d.%03d      ",gpsData.time.hour,gpsData.time.min,gpsData.time.sec,gpsData.time.thousand);
				} else { //If there is no GPS data, use the RTC time
					if(RTC_Set && (accel_read_complete || batt_read_complete)){
						log_data_index += (int)sprintf( &(log_data[log_data_index++]), "%02lu:%02lu:%02lu (%lu)  ",accel_hour,accel_min,accel_sec, accel_ctc);
					} else {
  					    for(i=0; i < 17; i++)
  						    log_data[log_data_index++]=' ';
					}
				}
				log_data[log_data_index++]='\t';

				//Log Acceleration
				if(accel_read_complete==1){
					log_data_index += (int) sprintf(log_data+log_data_index, "%d\t%d\t%d\t", acceleration_x+offsetX, acceleration_y+offsetY, acceleration_z+offsetZ );
				} else {
				   log_data[log_data_index++]='\t';
				   log_data[log_data_index++]='\t';
				   log_data[log_data_index++]='\t';
				}

				//Log Battery Level
				if(batt_read_complete==1){
					log_data_index += (int) sprintf(log_data+log_data_index, "%d", battery_level );
				}
				log_data[log_data_index++]='\t';

				if(1==valid_gps_data) {
					log_data[log_data_index++]=gpsData.fix;
					log_data[log_data_index++]='\t';
					if ( 0 != gpsData.fix ) {
						log_data_index += (int) sprintf( &(log_data[log_data_index++]), "%02d.%05d",gpsData.latitude.degree,gpsData.latitude.fractional);
						log_data[log_data_index++]='\t';
						log_data_index += (int) sprintf( &(log_data[log_data_index++]), "%02d.%05d",gpsData.longitude.degree,gpsData.longitude.fractional);
						log_data[log_data_index++]='\t';
						if ( SHRT_MAX != gpsData.altitude )
							log_data_index += (int) sprintf( &(log_data[log_data_index++]), "%.1f",(float)(gpsData.altitude)/(float)10.0);
						log_data[log_data_index++]='\t';
						if ( SHRT_MAX != gpsData.hdop )
							log_data_index += (int) sprintf( &(log_data[log_data_index++]), "%.1f",(float)(gpsData.hdop)/(float)10.0);
						log_data[log_data_index++]='\t';
						log_data_index += (int) sprintf( &(log_data[log_data_index++]), "%d",gpsData.numSatellites);
					} else {
						log_data[log_data_index++]='\t';
						log_data[log_data_index++]='\t';
						log_data[log_data_index++]='\t';
					}
				} else {
					log_data[log_data_index++]='\t';
					log_data[log_data_index++]='\t';
					log_data[log_data_index++]='\t';
					log_data[log_data_index++]='\t';
				}
				log_data[log_data_index++]='\n';
				//Only Save Data if the buffer is full!
				if(log_data_index >= MAX_BUFFER_SIZE){
					VICIntEnClr =  UART0_INT | EINT1_INT;
					GreenOn();
					saveData(&LOG_FILE, log_data, log_data_index);
					log_data_index=0;
					GreenOff();
					VICIntEnable =  UART0_INT | EINT1_INT;
				}

			}
#endif
		    valid_gps_data=0;
		    final_gps_message_size=0;
		    accel_read_complete=0;
		    batt_read_complete=0;

		    VICIntEnable = TIMER0_INT | UART0_INT | EINT1_INT;
	    }
	}

    return 0;
}

//Usage: bootUp();
//Inputs: None
//This function initializes the serial port, the SD card, the I/O pins and the interrupts
void bootUp(void)
{
	//Initialize UART for RPRINTF
    rprintf_devopen(putc_serial0); //Init rprintf
	init_serial0(baud_rate0);
    delay_ms(10); //Delay for power to stablize

    //Bring up SD and FAT
    if(!sd_raw_init())
    {
        rprintf("SD Init Error\n");
        reset();
    }
    if(openroot())
    {
        rprintf("SD OpenRoot Error\n");
        reset();
    }

	//Initialize I/O Ports and Peripherals
	IODIR0 = BT_RTS | SSEL1 | BT_EN | ACCEL_CS1;
	IODIR0 &= ~(BT_CTS | VLIPO_MEAS | ACCEL_INT1 | ACCEL_INT2);

	IODIR1 = STAT0 | STAT1 | FLASH_CS | CD1 | CD2 | GPS_EN;
	IODIR1 &= ~BT_CON;

	//Initialize the SPI bus
	SPI0_Init();			//Select pin functions for SPI signals.
    S0SPCCR = 64;           // SCK = 1 MHz (60MHz / 64 ~= 1Mhz)
    S0SPCR  = 0x20;         // Master, no interrupt enable, 8 bits

	PINSEL0 |= (3<<6);	     //Set P0.3 to EINT1
	VICIntEnClr = EINT1_INT;//Make sure EINT1 interrupts are disabled

	EXTINT |= (1<<1);		//Clear the EINT1 Interrupt bit
	EXTMODE |= (1<<1);		//Set EINT1 to be edge sensitive
	EXTINT |= (1<<1);		//Clear the EINT1 Interrupt bit
	EXTPOLAR |= (1<<1);	//Set EINT1 to detect rising edges
	INTWAKE |= (1<<1);		//ARM will wake up from power down on an EINT1 interrupt
	EXTINT |= (1<<1);		//Clear the EINT1 Interrupt bit

	//Enable AD conversion on P0.22(AD1.7) FOR VLIPO_MEAS
	PINSEL1 &= ~(3<<12);
	PINSEL1 |= (1<<12);

	//Disable USB LED functionality
	PINSEL1 &= ~(3<<31);


	//Turn Off LED's
	GreenOff();
	BlueOff();

	//Make sure Accelerometer is NOT selected
	UnselectAccel();

	VPBDIV=1;					// Set PCLK equal to the System Clock

    VICIntEnClr = 0xFFFFFFFF;           // clear all interrupts
    VICIntSelect = 0x00000000;            // clear all FIQ selections
  //  VICDefVectAddr = (uint32_t)reset;     // point unvectored IRQs to reset()


 	//Enable Interrupts
    //Select IRQ Interrupt type for our interrupts
//	VICIntSelect = ~(UART0_INT | TIMER0_INT | RTC_INT);
	VICIntSelect = ~(UART0_INT | TIMER0_INT | EINT1_INT);

	VICVectCntl0 = 0x20 | 6;						//Set up the UART0 interrupt
	VICVectAddr0 = (unsigned int)RxData0ISR;
//	VICVectCntl1 = 0x20 | 13;						//Set up the RTC interrupt
//	VICVectAddr1 = (unsigned int)ISR_RTC;
	VICVectCntl2 = 0x20 | 4;						//Timer 0 Interrupt
	VICVectAddr2 = (unsigned int)timer0ISR;
	VICVectCntl3 = 0x20 | 15;						//EINT1 External Interrupt
	VICVectAddr3 = (unsigned int)EINT1ISR;

	//Setup the UART0 Interrupt
	U0IER = 0x01;				//Enable FIFO on UART with RDA interrupt (Receive Data Available)
	U0FCR &= 0x3F;				//Enable FIFO, set RDA interrupt for 1 character

	//Disable Bluetooth and Gps
	IOCLR0 = BT_EN;
	IOCLR0 = GPS_EN;

    //unselect the accelerometer
    UnselectAccel();
}

void initializeAccelerometer( char range ) {
	//Initialize the accelerometer
	initAccel(range);

	//Set Accel. to Interrupt.
	//WriteLis302Register(FF_WU_CFG_1, XHIE | ZHIE | YHIE);	//Enable LIS302 Accel. Interrupts
	//WriteLis302Register(FF_WU_THS_1, 55);					//Set Accelerometer Threshold to ~1G


	WriteLis302Register(CLICK_CFG, LIR | Double_X | Double_Y | Double_Z);	//Enable LIS302 Click. Interrupts
	WriteLis302Register(CLICK_THSY_X, 0xff );
	WriteLis302Register(CLICK_THSZ, 0xff );
	WriteLis302Register(CLICK_timelimit, 0x5 );
	WriteLis302Register(CLICK_latency, 0xf5 );
	WriteLis302Register(CLICK_window, 0xb5 );

	WriteLis302Register(Ctrl_Reg3, I1CFG0|I1CFG1|I1CFG2 );

	//Make sure Accelerometer is not selected
	UnselectAccel();
}

void initializeTimer( int freqDivider ) {
	//Setup the Timer0 Interrupt
	T0TC = 0; // clear the timer count
	//Set the PrescaleRegister for Timer 0 the maximum value for the Prescale Counter
	T0PR = 1200;					//Divide Clock(60Mhz) by 1200 for 50kHz PS

	T0TCR |=0X01;				//Timer Counter and Prescale Counter Enabled
	T0CTCR=0;					//Count Control Register  set in Timer Mode
	T0MCR=0x0003;				//Interrupt, Reset, and Stop Timer on Match

    //Set the Match Register Counter to the configured value
    //When this value will match the Counter value the actions configured in T0MCR
    //will be triggered
	if ( 0 < freqDivider ) {
  	    T0MR0=(50000/freqDivider);	//Divide Timer(100Khz) by freqDivider for ints on 10Hz
    } else {
	    T0MR0=5000; //Divide Timer(100Khz) by 10000 for ints on 10Hz
	}
	//Setup the RTC
	CCR = (1<<4);		//Select external 32.768 KHz clock for RTC(Keep RTC Disabled for now!)
}


//Usage: None (Automatically Called by FW)
//Inputs: None
//This function is a global interrupt called by a match on the Timer 1 match.  The interrupt
// is responsible for getting the current accelleration readings.
static void timer0ISR(void)
{
    if (0<accel_logging ) {
	    acceleration_x = (signed char) accelX();
	    acceleration_y = (signed char) accelY();
	    acceleration_z = (signed char) accelZ();
		//Save the time when the collection was performed
		accel_ctc  = CTC;
		accel_sec  = SEC;
		accel_min  = MIN;
		accel_hour = HOUR;
	    accel_read_complete=1;
	}
	if (ACTIVE==batt_logging) {
	    battery_level = ((get_adc_1(0x80)*3300)/1024)*2;	//Report Battery voltage in mV.
	    batt_read_complete=1;
	}
	T0IR = 0xFF;						//Clear the timer interrupt
	VICVectAddr = 0x00000000;			//Update the VIC priorities
}

static void EINT1ISR(void){
	unsigned char val=0;

	VICIntEnClr = EINT1_INT;	//Temporarily disable EINT2 Interrupts
	EXTINT |= (1<<1);			//Clear the interrupt bit in EINT1

    SelectAccel();
    SPI0_send(READ | CLICK_SRC);
    val=(unsigned char) SPI0_recv();
    UnselectAccel();

    if ( 0x40 & val ) {
	    if ( ACTIVE==sleepMode ) {
		    sleepMode=INACTIVE;
			BlueOff();
            VICIntEnClr = TIMER0_INT | UART0_INT;

            disable_gps_rmc_msgs();
			gps_message_index=0;//discard any RMC data left in the queue
            if ( 0 < gps_logging ) {
				enable_gps_gga_msgs((unsigned char)gps_logging);
			}
			VICIntEnable = TIMER0_INT | UART0_INT;
			GreenOn();
		}
		else {
		    sleepMode=ACTIVE;
			VICIntEnClr = TIMER0_INT | UART0_INT;
			GreenOn();
			if ( NULL != BINARY_FILE ) saveData(&BINARY_FILE, binary_data, binary_data_index);
			binary_data_index=0;

#ifndef BINARY_ONLY
			if ( NULL != LOG_FILE ) saveData(&LOG_FILE, log_data, log_data_index);
			log_data_index = 0;
			if ( NULL != NMEA_FILE ) saveData(&NMEA_FILE, nmea_data, nmea_data_index);
			nmea_data_index=0;
#endif
			GreenOff();
			//If we are configured to receive Gps disabled GGA
			//and nable RMC it while in sleepMode. to synch the local clock
			//This will also be used to have a slowly blinking blue led
            if ( 0 < gps_logging ) {
                disable_gps_gga_msgs();
				gps_message_index=0;
			}
            enable_gps_rmc_msgs(3); //3 sec interval
			VICIntEnable = TIMER0_INT |UART0_INT;
			BlueOn();
		}
    }

	VICIntEnable =  EINT1_INT;	//Re-enable the EINT1 Interrupts

	VICVectAddr = 0x00000000;		//Update the VIC priorities
}


//When the EOL character is found the message is marked as complete for further processing.
static void RxData0ISR(void)
{

   char val = (char) U0RBR;
    if (( val == '\n') || (gps_message_index>=GPS_BUFFER_SIZE)) {

		gps_message_complete=1;
		gps_message_size=gps_message_index;
		gps_message_index=0;
	}
	else{
	    if ((0x19 < val ) && (0x7f > val)) {
            if (0x24 == val ) gps_message_index = 0; //Restart if $ found
     	    gps_message[gps_message_index++]= val;
		    gps_message[gps_message_index]='\0';
		}
	}
	VICVectAddr = 0x00000000;			//Update the VIC priorities
}

void reset(void)
{
    // Intentionally fault Watchdog to trigger a reset condition
    WDMOD |= 3;
    WDFEED = 0xAA;
    WDFEED = 0x55;
    WDFEED = 0xAA;
    WDFEED = 0x00;
}

void sendParsedData(void){
	rprintf("Time: %02d:%02d:%02d",gpsData.time.hour,gpsData.time.min,gpsData.time.sec);
	rprintf("\n\r");
	rprintf("Latitude: ");
	rprintf("%d.%05d", gpsData.latitude.degree,gpsData.latitude.fractional );
	rprintf("Longitude: ");
	rprintf("%d.%05d", gpsData.longitude.degree, gpsData.latitude.fractional);
	rprintf("Altitude: ");
	rprintf("%d", gpsData.altitude);
	rprintf("\n\r");
	rprintf("Fix: %c\n\r", gpsData.fix);
}

//Usage: accel = get_adc_1(CHANNEL);
//Inputs: int channel - integer corresponding to the ADC channel to be converted
//Outputs: None
//Description: Returns the raw analog to digital conversion of the input channel.
int get_adc_1(char channel)
{
    int val;
    AD1CR = 0;
    AD1GDR = 0;

    //Change suggested by PausingReality
    //This will result in a 4MHz A/D clock (60 / (14 + 1)).
    //AD1CR = 0x00200600 | channel;
    AD1CR = 0x00200E00 | channel;

    AD1CR |= 0x01000000;
    do
    {
        val = AD1GDR;                   // Read A/D Data Register
    }
    while ((val & 0x80000000) == 0);  //Wait for the conversion to complete
    val = ((val >> 6) & 0x03FF);  //Extract the A/D result

    return val;
}

//void saveData(char *buf, int buf_size)
void saveData(struct fat16_file_struct **fd, const char * const buf, const int buf_size)
{
    if ( 0 < buf_size ) {
        if ( NULL != *fd ) {
		    fat16_write_file(*fd, (const unsigned char *const)buf, buf_size);
	        sd_raw_sync();
		}
	}
}

signed int atoi(const char *const s){
    const char *end;
    const char *start = s;
    int mult = 1;
    int sign = 1;
    int val = 0;
    while( *start == ' ' ) start++;
    if ( '-' == *start ) { start++; sign = -1;}
    else if ( '+' == *start ) { start++; sign = 1;}
    end = start;
    while( *end >= '0' && *end <= '9' ) end++;
    end--;
    while( end >= start ) {
        val += (int)((*end)-'0')*mult;
        mult *= 10;
        end--;
    }
    return val*sign;
}

void configBluetooth(void){
	//Enter Command Mode
	delay_ms(1000);
	rprintf("$$$");
	delay_ms(1000);

	//Disable the Config Timer
	rprintf("ST,0\r");
	delay_ms(50);

	//Activate Fast Data Mode
	rprintf("SN,KinetaMap\r");
	delay_ms(50);

	//Activate Fast Data Mode
	rprintf("F,1\r");
	delay_ms(50);

	//Exit Command Mode
	//rprintf("---");
	//delay_ms(50);
}

#define CONFIG_LINE1 "config_menu=1\nlog_gps=5\nlog_accel=10\nsend_gps_to_bt=1\nlog_batt=0\n"
#define CONFIG_LINE2 "save_txt=1\nsave_binary=0\nsave_nmea=0\naccel_X=0\naccel_Y=0\naccel_Z=0\ng_FSrange=1\n"
#define DATA_FILE_HEADER "Time               \tX\tY\tZ\tBatt\tFix\tLatitude\tLongitude\tAlt\tHDOP\tNum Satell\n"

void createLogFile(void){
    char name[32] = { '\0' };
    int file_number = 0;

#ifndef BINARY_ONLY
    if (ACTIVE==txt_logging) {
    	sprintf(name, "KinetaMap%03d.tsv", file_number);

        while(root_file_exists(name))
        {
            file_number++;
            if(file_number == 250)
            {
                rprintf("\nToo many files in root!\n");
            }
            sprintf(name, "KinetaMap%03d.tsv", file_number);
        }

        LOG_FILE = root_open_new(name);
	    fat16_write_file(LOG_FILE, (unsigned char*)DATA_FILE_HEADER, strlen(DATA_FILE_HEADER));
    	sd_raw_sync();
	}

    if (ACTIVE==nmea_logging) {
	    name[0] = '\0';
		sprintf(name, "km-nmea%03d.txt", file_number);
		//Possibily a bug in this API, returns true even if the file
		//doesn't exist. Calling twice seems to reset it.
        root_file_exists(name);

		while(root_file_exists(name))
		{
			file_number++;
			if(file_number == 250)
			{
				rprintf("\nToo many files in root!\n");
			}
     	    name[0] = '\0';
			sprintf(name, "km-nmea%03d.txt", file_number);
		}
		NMEA_FILE = root_open_new(name);
    	sd_raw_sync();
	}
#endif
	if (ACTIVE==binary_logging ) {
	    name[0] = '\0';
		sprintf(name, "km-bin%03d.dat", file_number);
		//Possibily a bug in this API, returns true even if the file
		//doesn't exist. Calling twice seems to reset it.
        root_file_exists(name);

		while(root_file_exists(name))
		{
			file_number++;
			if(file_number == 250)
			{
				rprintf("\nToo many files in root!\n");
			}
    	    name[0] = '\0';
			sprintf(name, "km-bin%03d.dat", file_number);
		}

		BINARY_FILE = root_open_new(name);
    	sd_raw_sync();
	    //Simple Signature on the binary file
		binary_data[binary_data_index++] = 'K';
		binary_data[binary_data_index++] = 'M';
		binary_data[binary_data_index++] = '0';
		binary_data[binary_data_index++] = '1';

		binary_data[binary_data_index++] = 'C';
		binary_data[binary_data_index++] = offsetX;
		binary_data[binary_data_index++] = offsetY;
		binary_data[binary_data_index++] = offsetZ;
	}
}

//Loads parameters from the configuration file.
//Very primitive handling of the config file, it expects the
//parameter in a fixed order and with the value immediately
//after the =.

#define CONFIG_FILE "kineta_config.txt"
void load_settings(void){
	struct fat16_file_struct * settingsFile;
    char *settingsString = binary_data; //binary_data is not used at this point at it is large enough

	char current_character;
	int i, fileLen;

	config_menu_setting=ACTIVE;
    gps_logging   = 5;
	accel_logging = 10;
	batt_logging  = INACTIVE;
	gps_over_bt   = ACTIVE;
	txt_logging   = INACTIVE;
	binary_logging= INACTIVE;
	nmea_logging  = INACTIVE;

	offsetX = 0;
	offsetY = 0;
	offsetZ = 0;

	gRange  = 1;//Full Scale

	//Open the configuration File
	settingsFile=root_open(CONFIG_FILE);
	//Get the contents of the configuration file into the "settingsString"
	if(settingsFile != NULL){
		fileLen=0;
		while(fat16_read_file(settingsFile, (uint8_t*) &current_character, (uint16_t)1)>0){
			settingsString[fileLen++]=current_character;
			if (fileLen>BINARY_BUFFER_SIZE) {fileLen--; break;}
		}
  	    //settingsString is now filled with configuration settings.
	    //See KinetaMapPins.h for description of Configuration File
	    fat16_close_file(settingsFile);

		//Load the settings
		i = 0;
		while( i < fileLen ) {
			if (settingsString[i] == '=') {
				i++;
				if(settingsString[i]=='1')config_menu_setting=ACTIVE;
				else config_menu_setting=INACTIVE;
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				gps_logging=(unsigned char) atoi(&settingsString[i]);
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				accel_logging=atoi(&settingsString[i]);
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				if(settingsString[i]=='1')gps_over_bt=ACTIVE;
				else gps_over_bt=INACTIVE;
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				if(settingsString[i]=='1')batt_logging=ACTIVE;
				else batt_logging=INACTIVE;
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				if(settingsString[i]=='1')txt_logging=ACTIVE;
				else txt_logging=INACTIVE;
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				if(settingsString[i]=='1')binary_logging=ACTIVE;
				else binary_logging=INACTIVE;
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				if(settingsString[i]=='1')nmea_logging=ACTIVE;
				else nmea_logging=INACTIVE;
				break;
			}
			i++;
		}

		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				offsetX = (signed char) atoi( &settingsString[i] );
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				offsetY = (signed char) atoi( &settingsString[i] );
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				offsetZ = (signed char) atoi( &settingsString[i] );
				break;
			}
			i++;
		}
		while( i < fileLen) {
			if (settingsString[i] == '=') {
				i++;
				if(settingsString[i]!='0')gRange=1;
				else gRange = 0;
				break;
			}
			i++;
		}
    } else {
		//If the config file doesn't exist, then create it!
		if(!root_file_exists(CONFIG_FILE)){
			settingsFile = root_open_new(CONFIG_FILE);
			fat16_write_file(settingsFile, (uint8_t*)CONFIG_LINE1, strlen(CONFIG_LINE1));
			fat16_write_file(settingsFile, (uint8_t*)CONFIG_LINE2, strlen(CONFIG_LINE2));
			sd_raw_sync();
			fat16_close_file(settingsFile);
		}
	}
}

void initializeBluetooth(void){
	//Initialize the Bluetooth device!
    rprintf_devopen(putc_serial1); 	//Init rprintf on UART1 (Bluetooth Channel)
	init_serial1(baud_rate1);		//Set bluetooth Baud Rate
	IOSET0 = BT_EN;			//Turn on the Bluetooth device
	delay_ms(200);
	configBluetooth();
}

void initializeGps(void){
	//Initialize the GPS receiver
	IOSET1 = GPS_EN;
	delay_ms(200);

	disable_all_gps_msgs();
	delay_ms(200);

}

int configMenu(void){
	int selection=0, error_code=1;
	char settingsString[100];
	char character, tempbuffer[50];
	struct fat16_file_struct* fd;

	rprintf("----------- KinetaMap Config Menu -----------\n");
	rprintf("              FW Version  1.1.2\n");
	rprintf("            SparkFun  Electronics\n");
	rprintf("_____________________________________________\n");
	rprintf("1) Read directory\n");
	rprintf("2) Read file\n");
	rprintf("3) Delete file\n");
	rprintf("4) Format SD card\n");
	rprintf("5) Listen to GPS (press any key to stop)\n");
	rprintf("6) Check Accelerometer(press any key to stop)\n");
	rprintf("7) Change settings in the config file\n");

	rprintf("9) Exit menu and start logging\n");

	selection=getc1();
	rprintf("\nOption = %c\n\n", selection);

	if(selection=='1'){
		rootDirectory_files_stream(1);
		rprintf("Files in Root Directory:\n");
		character = 1;
		while (character != 0)
		{
			character = rootDirectory_files_stream(0);

			if (character == ',')
			{
				putc_serial1('\n');
				putc_serial1('\r');
			}
			else putc_serial1(character);
		}
		rprintf("\n");
		character = 0;
	}
	if(selection=='2'){
		for (int m = 0; m < 50; m++) tempbuffer[m] = 0;
		rprintf("Enter the name of the file you wish to read: ");
		for (int m = 0; m < 50; m++)
		{
			character = getc1();

			if ((character == 10) | (character == 13)) break;
			putc_serial1(character);
			tempbuffer[m] = character;
		}

		fd = root_open((char*)tempbuffer);
		if (fd == 0)
		{
			rprintf("\nFailed to open file.\n\n");
			return 0;
		}

		rprintf("\n\nFile Contents:\n");
		while(error_code > 0)
		{
			tempbuffer[0] = 0;

			error_code=fat16_read_file(fd, (uint8_t*)tempbuffer, 1);
			if(error_code<0){
				rprintf("\n\nFailed to read %d\n\n",character);
				return 0;
			}

			if (tempbuffer[0] != 0) putc_serial1(tempbuffer[0]);
			if(tempbuffer[0]=='\n')putc_serial1('\r');
		}

		fat16_close_file(fd);
		sd_raw_sync();

		rprintf("\nEnd.\n\n");
	}
	if(selection=='3'){
		for (int m = 0; m < 50; m++) tempbuffer[m] = 0;
		rprintf("Enter the name of the file you wish to delete: ");
		for (int m = 0; m < 50; m++)
		{
			character = getc1();

			if ((character == 10) | (character == 13)) break;
			putc_serial1(character);
			tempbuffer[m] = character;

		}

		character = root_delete((char*)tempbuffer);
		if (character == 0) rprintf("\nFile Deleted.\n\n");
		else rprintf("\nFailed.\n\n");

		sd_raw_sync();
	}
	if(selection=='4'){
		root_format();
		rprintf("Card Formatted\n\n");
	}
	if(selection=='5'){
		while(!(U1LSR & (1<<0))){
			if(U0LSR & (1<<0))rprintf("%c", U0RBR); //If there is a character in the Rx buffer, print it out!
		}
		rprintf("\n");
	}
	if(selection=='6'){
		while(!(U1LSR & (1<<0))){
			rprintf("X=%d\tY=%d\tZ=%d\n", (signed char) accelX(),(signed char) accelY(),(signed char) accelZ());
			delay_ms(500);
		}
		rprintf("\n\n");
	}
	if(selection=='7'){
		char option;

		if(root_file_exists(CONFIG_FILE))root_delete(CONFIG_FILE);
		rprintf("\nConfiguration File Deleted!\n\n");
		rprintf("\nPls edit the configuration file for further customization.n\n");

		rprintf("Enable Configuration Menu(y/n)?\n");
		option=getc1();
		rprintf("%c\n\n", option);
		if(option=='y'||option=='Y')config_menu_setting=ACTIVE;
		else config_menu_setting=INACTIVE;

		rprintf("Log GPS Data(y/n)?\n");
		option=getc1();
		rprintf("%c\n\n", option);
		if(option=='y'||option=='Y')gps_logging=1;
		else gps_logging=-1;

		rprintf("Log Accelerometer Data(y/n)?\n");
		option=getc1();
		rprintf("%c\n\n", option);
		if(option=='y'||option=='Y')accel_logging=10000;
		else accel_logging=-1;

		rprintf("Log Battery Level(y/n)?\n");
		option=getc1();
		rprintf("%c\n\n", option);
		if(option=='y'||option=='Y')batt_logging=ACTIVE;
		else batt_logging=INACTIVE;

		rprintf("Send GPS data over Bluetooth(y/n)?\n");
		option=getc1();
		rprintf("%c\n\n", option);
		if(option=='y'||option=='Y')gps_over_bt=ACTIVE;
		else gps_over_bt=INACTIVE;

		fd = root_open_new(CONFIG_FILE);
		sprintf(settingsString, "config_menu=%d\nlog_gps=%d\nlog_accel=%d\nsend_gps_to_bt=%d\nlog_batt=%d\n",
			config_menu_setting, gps_logging, accel_logging, gps_over_bt, batt_logging);
		fat16_write_file(fd, (uint8_t*)settingsString, strlen(settingsString));
		fat16_write_file(fd, (uint8_t*)CONFIG_LINE2, strlen(CONFIG_LINE2));
		sd_raw_sync();
		fat16_close_file(fd);

	}
	if(selection=='9')return selection;
	return 0;
}
