#include <ctype.h>
#include "parseGps.h"

int parseTime( char *line, Time *time ) {
    //0123456789
	//205659.000

    if ( ',' == line[0]) {
		time->hour = UCHAR_MAX;
		time->min  = UCHAR_MAX;
		time->sec  = UCHAR_MAX;
		return 0;
	}
	if ( isdigit(line[0]) && isdigit(line[1]) && isdigit(line[2]) && isdigit(line[3]) && isdigit(line[4]) && isdigit(line[5]) &&
	     ('.' == line[6]) && isdigit(line[7]) && isdigit(line[8]) && isdigit(line[9]) ) {
        time->hour = (unsigned char)((line[1]-'0')+(line[0]-'0')*10);
        time->min  = (unsigned char)((line[3]-'0')+(line[2]-'0')*10);
        time->sec  = (unsigned char)((line[5]-'0')+(line[4]-'0')*10);

        time->thousand = (unsigned short)((line[9]-'0')+(line[8]-'0')*10+(line[7]-'0')*100);

        return 10;
	}
    return -1;
}

int parseDate( char *line, Date *date ) {
    //012345
	//231108

    if ( ',' == line[0]) {
		date->day   = UCHAR_MAX;
		date->month = UCHAR_MAX;
		date->year  = UCHAR_MAX;
		return 0;
	}
	if ( isdigit(line[0]) && isdigit(line[1]) && isdigit(line[2]) && isdigit(line[3]) && isdigit(line[4]) && isdigit(line[5])) {
        date->day = (unsigned char)((line[1]-'0')+(line[0]-'0')*10);
        date->month  = (unsigned char)((line[3]-'0')+(line[2]-'0')*10);
        date->year  = (unsigned char)((line[5]-'0')+(line[4]-'0')*10);
        return 10;
	}
	return -1;
}

int parseLat( char *line, Coord *coord ) {
	//012345678
	//3327.4862
	if ( ',' == line[0] ) return 0;

	if ( isdigit(line[0]) && isdigit(line[1]) && isdigit(line[2]) && isdigit(line[3]) &&
	     ('.' == line[4]) && isdigit(line[5]) && isdigit(line[6]) && isdigit(line[7]) && isdigit(line[8])) {
        coord->degree = (unsigned char)((line[1]-'0')+(line[0]-'0')*10);
        coord->fractional = (unsigned int)(((line[8]-'0')+(line[7]-'0')*10+(line[6]-'0')*100+(line[5]-'0')*1000+(line[3]-'0')*10000+(line[2]-'0')*100000)/6);
        return 9;
	}
	return -1;
}
int parseLong( char *line, Coord *coord ) {
	//0123456789
	//11149.9902
	if ( ',' == line[0] ) return 0;

	if ( isdigit(line[0]) && isdigit(line[1]) && isdigit(line[2]) && isdigit(line[3]) && isdigit(line[4]) &&
	     ('.' == line[5]) && isdigit(line[6]) && isdigit(line[7]) && isdigit(line[8]) && isdigit(line[9])) {
		coord->degree = (unsigned char)((line[2]-'0')+(line[1]-'0')*10+(line[0]-'0')*100);
		coord->fractional = (unsigned int)(((line[9]-'0')+(line[8]-'0')*10+(line[7]-'0')*100+(line[6]-'0')*1000+(line[4]-'0')*10000+(line[3]-'0')*100000)/6);
	    return 10;
	}
	return -1;
}

int parseRmc( char *line, const int length, Date *date, Time *time, Coord *latitude, Coord *longitude, char *const fix ) {
	//$GPRMC,205659.000,A,3327.4862,N,11149.9902,W,60.41,83.87,231108,,
    //$GPRMC,171615.263,V,,,,,,,011208,,
    int i;
	char *start = line;
	*fix = 0;

	if ( ',' != *start ) return -1;
	start++;
    i = parseTime( start, time	);
    if ( -1 == i ) return -1;
	start += i+1;
    if ((start-line)>length ) return -1;

    if ( 'A' == *start ) {*fix = 1;start++;}
    else if ( 'V' == *start ) {*fix = 0;start++;}
	else if ( '.' == *start ) {*fix = 0;}
    start++;
    if ((start-line)>length ) return -1;

    i = parseLat( start, latitude	);
    if ( -1 == i ) return -1;
    start += i+1;
    if ((start-line)>length ) return -1;

    if ( 'S' == *start )  latitude->degree *= -1;
    else if ( 'N' == *start ) {}
    else if ( ',' == *start ) start--;
    start+=2;
    if ((start-line)>length ) return -1;

    i = parseLong( start, longitude );
    if ( -1 == i ) return -1;
    start += i+1;
    if ((start-line)>length ) return -1;

    if ( 'W' == *start )  longitude->degree *= -1;
    else if ( 'E' == *start ) {}
    else if ( ',' == *start ) start--;
    start+=2;
    if ((start-line)>length ) return -1;

    //Skip Speed Over Ground
    while ( ',' != *start ) start++;
    start++;
    if ((start-line)>length ) return -1;

    //Skip Course Over Ground
    while ( ',' != *start ) start++;
    start++;
    parseDate( start, date );

    return start-line;
}

int parseGga( char *line, const int length, Time *const time, Coord *const latitude, Coord *const longitude, unsigned char *const numSatellites, short *const hdop, short *const altitude, char *const fix ) {
	//$GPGGA,205700.000,3327.4878,N,11149.9699,W,2,09,1.0,368.4,M,-26.5,M,0.8,0000
	char *start = line;
	char *end = line;
	char *tmp;
	int i;
	*fix = 0;
	if ( ',' != *start ) return -1;
	start++;
    i = parseTime( start, time	);
    if ( -1 == i ) return -1;
	start += i+1;
    if ((start-line)>length ) return -1;

    i = parseLat( start, latitude	);
    if ( -1 == i ) return -1;
    start += i+1;
    if ((start-line)>length ) return -1;

    if ( 'S' == *start )  latitude->degree *= -1;
    else if ( 'N' != *start ) return 0;
    start+=2;
    if ((start-line)>length ) return -1;

    i = parseLong( start, longitude );
    if ( -1 == i ) return -1;
    start += i+1;
    if ((start-line)>length ) return -1;

    if ( 'W' == *start )  longitude->degree *= -1;
    else if ( 'E' != *start ) return 0;
    start+=2;
    if ((start-line)>length ) return -1;

    *fix = *start;
    if ( '0' == *start )  return -1; //No Fix
    start+=2;
    if ((start-line)>length ) return -1;

    *numSatellites = (unsigned char)((*(start+1)-'0')+(*start-'0')*10);
    start += 3;
    if ((start-line)>length ) return -1;

    //HDOP
	if ( *(start+1) != ',' ) {
        end = start; i = 0;
        while ( ',' != *end ) end++;
        tmp = end;
        while ( '.' != *tmp ) tmp--;
		//Make sure we just take one digit after the decimal point
		*hdop = *(tmp+1) - '0';
		tmp--;
		i = 10;
		while ( tmp >=start ) {
		    *hdop += (*tmp-- - '0')*i;
			i *= 10;
		}
	} else {
	    end = start+1;
	    *hdop = SHRT_MAX;
	}
	start = end+1;
    if ((start-line)>length ) return -1;

    //Altitude
	if ( *(start+1) != ',' ) {
        end = start; i = 0;
        while ( ',' != *end ) end++;
        tmp = end;
        while ( '.' != *tmp ) tmp--;
		//Make sure we just take one digit after the decimal point
		*altitude = *(tmp+1) - '0';
		tmp--;
		i = 10;
		while ( tmp >=start ) {
		    *altitude += (*tmp-- - '0')*i;
			i *= 10;
		}
	} else {
	    end = start+1;
	    *altitude = SHRT_MAX;
	}
	//start = end+1;

	//Ignore geoid separation


    return start-line;
}


int parseNmea( char *const line, const int stringLength, GPSdata *const gpsData ) {
    if ( '$' != line[0] || 'G' != line[1] || 'P' != line[2] ) return -1;
    if ( 'R' == line[3] && 'M' == line[4] && 'C' == line[5] ) {
		gpsData->hdop = 0; gpsData->altitude = 0; gpsData->numSatellites = 0;
        return parseRmc( &line[6], stringLength-6, &(gpsData->date), &(gpsData->time), &(gpsData->latitude), &(gpsData->longitude), &(gpsData->fix) );
	} else  if ( 'G' == line[3] && 'G' == line[4] && 'A' == line[5] ) {
		gpsData->date.day = UCHAR_MAX;
        return parseGga( &line[6], stringLength-6, &(gpsData->time), &(gpsData->latitude), &(gpsData->longitude), &(gpsData->numSatellites), &(gpsData->hdop), &(gpsData->altitude),&(gpsData->fix));
	} else {
		gpsData->fix = 0;
	}
	return -1;
}

